###Script for KM model#####
####Lib####
library(PSPManalysis)
library(ggplot2)
#####Settings#####
pars.names=c("Delta","Xmax1","Xmax2","LB","LV","LJ","H","BETA","AMAX","GAMMA","LM","AC",
             "MU","Psi","Amin","Meta","QS","RHO","Sj","SIGMA", "mus", "Tqs")
output = c("In1_L", "In2_L", "L", "Lm","L2","Lm2","J","Jm","A","Am")
Env = c("X1", "X2", "birthrate")
meta_index=which(pars.names == 'Meta')-1
Sj_index=which(pars.names == 'Sj')-1
psi_index <- which(pars.names == 'Psi')-1
#####Figure E2#####
Xmax1=0.05
Xmax2=0
Amax=6.66667
Amin=0.1*Amax
Sigma=30
Psi=0
Meta=0
Sj=5E-6
Mu=0.05
rho=0.5
pars_km=c(0.1,Xmax1,Xmax2,7,50,110,10,9E-3,Amax,0.006,300,0.003,Mu,Psi,Amin,Meta,0.2,rho,Sj,Sigma,0,0)

BPfind<-PSPMequi(modelname="KM_Meta", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_km,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_km[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="KM_Meta",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.01, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_km,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="KM_Meta",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 1), 
                   parameters=pars_km,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[15]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.01
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
pars_km=c(0.1,Xmax1,Xmax2,7,50,110,10,9E-3,Amax,0.006,300,0.003,Mu,Psi,Amin,Meta,0.2,rho,Sj,Sigma,0,0)

canonical_fig4 <- PSPMevodyn(modelname="KM_Meta", 
                             startpoint=c(startpoint, psi,Meta,Sj), 
                             curvepars=c(1,10000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,Sj_index,2E-6,20),
                             covars = NULL,
                             parameters = pars_km,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_fig4$curvepoints)[2:4]<-Env
colnames(canonical_fig4$curvepoints)[8:17]<-output
canonicalmeta_fig4<-as.data.frame(canonical_fig4$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_fig4$PSI[nrow(canonicalmeta_fig4)])
meta=canonicalmeta_fig4$Meta[nrow(canonicalmeta_fig4)]
Sj = max(canonicalmeta_fig4$LC[nrow(canonicalmeta_fig4)],2E-6)
startpnt=as.numeric(canonicalmeta_fig4[nrow(canonicalmeta_fig4), c(2:4)])
canonical_fig4B <- PSPMevodyn(modelname="KM_Meta", 
                              startpoint=c(startpnt, psi,meta,Sj), 
                              curvepars=c(100,1000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,Sj_index,2E-6,20),
                              covars = NULL,
                              parameters = pars_km,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_fig4B$curvepoints)[2:4]<-Env
colnames(canonical_fig4B$curvepoints)[8:17]<-output
canonicalmeta_fig4B<-as.data.frame(canonical_fig4B$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
Psi = max(0,canonicalmeta_fig4B$PSI[nrow(canonicalmeta_fig4B)])
Meta=canonicalmeta_fig4B$Meta[nrow(canonicalmeta_fig4B)]
Sj = canonicalmeta_fig4B$LC[nrow(canonicalmeta_fig4B)]
startpnt=as.numeric(canonicalmeta_fig4B[nrow(canonicalmeta_fig4B), c(2:4)])
pars_km=c(0.1,Xmax1,Xmax2,7,50,110,10,9E-3,Amax,0.006,300,0.003,Mu,Psi,Amin,Meta,0.2,rho,Sj,Sigma,0,0)

MetaESS1 <- PSPMequi(modelname="KM_Meta",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1), 
                     parameters=pars_km,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="KM_Meta",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1), 
                     parameters=pars_km,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:16]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$R0_xx_yy = MetaESS1_data$`R0_xx[15]` - MetaESS1_data$`R0_yy[15]`
MetaESS1_data$ESS = ifelse(is.na(MetaESS1_data$R0_xx_yy), 'ESS', ifelse(MetaESS1_data$R0_xx_yy<0, "ERP", ifelse(MetaESS1_data$`R0_yy[15]`<0, "CSS", "EBP")))

colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:16]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS2_data$R0_xx_yy = MetaESS2_data$`R0_xx[15]` - MetaESS2_data$`R0_yy[15]`
MetaESS2_data$ESS = ifelse(is.na(MetaESS2_data$R0_xx_yy), 'ESS', ifelse(MetaESS2_data$R0_xx_yy<0, "ERP", ifelse(MetaESS2_data$`R0_yy[15]`<0, "CSS", "EBP")))
MetaESS2_data = subset(MetaESS2_data, `R0_x[13]`<=0)

ggplot(data=MetaESS1_data,aes(x=Xmax2,y=Meta))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))

#Continue with psi in ESS 
pnt = c(0.07268943,
        0.01972135,
        0.05749812,
        5.060402e-06,
        0.9991020, psi)
MetaESS3 <- PSPMequi(modelname="KM_Meta",
                     biftype='ESS',
                     startpoint=pnt,
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1,0,psi_index,0,1), 
                     parameters=pars_km,
                     options=c("popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)


colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[7:16]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$ESS=ifelse(MetaESS3_data$`eig J`>0, "ERP", ifelse(MetaESS3_data$`eig H`<0&MetaESS3_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS3_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS3_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 


ggplot(data=MetaESS1_data,aes(x=Xmax2,y=Meta))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))+
  geom_path(data=MetaESS3_data,aes(colour=ESS))+xlim(c(0,1))
#Use canonical equation to check that erp is not an erp but css

#Step D: Find unstable equilibrium
#STart close to point where meta can evolve
psi = 0 
Sj = 5E-6
pnt = c(0.15398467,      0.03369506 ,     0.15068033 , 1.27691524E-06,      0.09480440)
MetaESS_un <- PSPMequi(modelname="KM_Meta",
                     biftype='ESS',
                     startpoint=pnt,
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1), 
                     parameters=pars_km,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)
MetaESS_un2 <- PSPMequi(modelname="KM_Meta",
                       biftype='ESS',
                       startpoint=pnt,
                       stepsize=-0.1, 
                       parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1), 
                       parameters=pars_km,
                       options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                                 "popEVO", "0", 'parEVO',as.character(Sj_index)),
                       clean=TRUE)
MetaESS_un_data=as.data.frame(MetaESS_un$curvepoints)
MetaESS_un_data$R0_xx_yy = MetaESS_un_data$`R0_xx[15]` - MetaESS_un_data$`R0_yy[15]`
MetaESS_un_data$ESS = ifelse(is.na(MetaESS_un_data$R0_xx_yy), 'ESS', ifelse(MetaESS_un_data$R0_xx_yy<0, "ERP", ifelse(MetaESS_un_data$`R0_yy[15]`<0, "CSS", "EBP")))
MetaESS_un_data2=as.data.frame(MetaESS_un2$curvepoints)
MetaESS_un_data2$R0_xx_yy = MetaESS_un_data2$`R0_xx[15]` - MetaESS_un_data2$`R0_yy[15]`
MetaESS_un_data2$ESS = ifelse(is.na(MetaESS_un_data2$R0_xx_yy), 'ESS', ifelse(MetaESS_un_data2$R0_xx_yy<0, "ERP", ifelse(MetaESS_un_data2$`R0_yy[15]`<0, "CSS", "EBP")))


ggplot(data=MetaESS1_data,aes(x=Xmax2,y=Meta))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))+
  geom_path(data=MetaESS3_data,aes(colour=ESS))+
  geom_path(data=MetaESS_un_data,aes(colour=ESS))+
  geom_path(data=MetaESS_un_data2,aes(colour=ESS))+
  xlim(c(0,1))

