##REMOVE###
setwd("~/Documents/Manuscripts/Metamorphosis/Metamorposis_R2/Code/")
########

###R-Script to reproduce figures of the paper 'The evolutionary ecology of metamorphosis'####
#To install the pspmanalysis package, use: devtools::install_bitbucket("amderoos/pspmanalysis/R")
#Use '?PSPMequi' and '?PSPMevodyn' for help

####Lib####
library(PSPManalysis)
library(ggplot2)

###Settings and names####
pars.names=c("Delta", "Xmax1", "Xmax2", "Amin", "Amax", 
             "Alpha", "W0", "Chi1", "Chi2", "Chi3", 
             "Chi4","M1", "M2", "K1", "K2",
             "X0", "Xmin", "Xj", "Xf", "Qj", 
             "Qa", "Qs",  "Mu0", "SIGMA", 
             "Psi", "Meta","Rho")
meta_index=which(pars.names == 'Meta')-1
xj_index=which(pars.names == 'Xj')-1
psi_index <- which(pars.names == 'Psi')-1
## Vector of names of output variables
#In = Intake, Ir= irreversible mass, Rev=Reversible mass
output = c("In1_L", "In2_L", "Ir_L", "Rev_L", "L","In1_L2", "In2_L2", "Ir_L2", "Rev_L2", "L2","In1_J", "In2_J", 
           "Ir_J", "Rev_J", "J","In1_A", "In2_A", "Ir_A", "Rev_A", "A", "Age_mat", "Maxage" )
Env=c("X1", "X2", "Birthrate")

###Standard parameters####
xmin <- 1
meta <- 0
rho <- 0.5
qs <- 0.2
Xmax1 <- 5
alpha <- 0.6
####
#####Attack rate function######
#Calculate the attack rate as a function of size for different values of Meta, psi and Xj
#needs a vector 'sizedata' with the sizes for which to calculate the attack rates
attackrates <-function (meta,psi,Xj,sizedata,dataframe=NULL) {
  amax=100000
  amin=0.1*amax
  alpha = 0.6
  w0=17.42
  wmin = 1.742
  
  newdata=data.frame(size=sizedata, a1=seq(1,length(sizedata)), a2=seq(1,length(sizedata)))
  
  for (i in 1:length(sizedata)) {
    w=sizedata[i]
    if(w>Xj) {
      psi_2 =min(1, psi+meta)
    }
    else {psi_2=psi}
    A1=(1-psi_2)*(amax-amin)+amin
    A2=psi_2*(amax-amin)+amin
    a1 = A1*((w/w0)*exp(1-(w/w0)))^alpha
    a2 = ifelse(w<=wmin, 0, A2*(((w-wmin)/w0)*exp(1-((w-wmin)/w0)))^alpha)
    if (w>Xj) {
      newdata$a1pre[i] =  NA
      newdata$a2pre[i] =  NA
      newdata$a1post[i] =  a1
      newdata$a2post[i] =  a2
    }
    else {
      newdata$a1pre[i] =  a1
      newdata$a2pre[i] =  a2
      newdata$a1post[i] =  NA
      newdata$a2post[i] =  NA
    }
  }
  assign(dataframe,newdata, envir = .GlobalEnv)
}
###########
####Code for Figure 1: Attack rates as a function of effective body mass#####

sizedata=c(seq(0.000804*1.742,0.99*1.742,0.01),seq(0.99*1.742,1.01*1.742,0.0001),seq(1.01*1.742,150*1.742,0.01))
###No meta, full specialization on juv resource
attackrates(Xj=1.742,meta=0,psi=0,sizedata=sizedata,dataframe='attacknometa')
ggplot(data=attacknometa,aes(x=size,y=a1pre))+geom_line((aes(colour='a1')),size=1)+
  geom_line(aes(y=a1post,colour='a1'),size=1)+
  geom_line(aes(y=a2pre,colour='a2'),size=1)+geom_line(aes(y=a2post,colour='a2'),size=1)
##No meta, full specialization on adult resource
attackrates(Xj=1.742,meta=0,psi=1,sizedata=sizedata,dataframe='attacknometa_adspec')
ggplot(data=attacknometa_adspec,aes(x=size,y=a1pre))+geom_line((aes(colour='a1')),size=1)+
  geom_line(aes(y=a1post,colour='a1'),size=1)+
  geom_line(aes(y=a2pre,colour='a2'),size=1)+geom_line(aes(y=a2post,colour='a2'),size=1)


################
####Code for Figure 2: Evolution of specialization#####
Xj <- 0.001 #Value doesn't matter, since there is no metamorphosis
mutrate <- 1 ##=mutrate*inverse of offspring variance *var in mut
var <- 1
#Change Xmax2 (values used: 1,2,6,10,20) to calculate evolutionary trajectories for different supply rates
#Change stepsize in BPfind to a negative value and start with a high initial r1max value in case bp can not be found
Xmax2 <- 10 #1/2/6/10/20
##Initially no specialization psi = 0.05, initially full specialization psi = 1 (For Xmax2=1 psi is initially 0.9 since there is no viable population for psi=1 and Xmax2=1)
psi <- 1 #1/0.05
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
                alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
                0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
                0.742,1,qs,0.01,10,psi,meta,rho)
#Step 1: Starting from the trivial equi, find the bp from where to continue the non-trivial equilibrium 
# Make stepsize negative if bp is not found
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step 3: Canonical equation of adaptive dynamics
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
canonicalpsi_CA <- PSPMevodyn(modelname="Metamorphosis_model", 
                               startpoint=c(startpnt, psi), 
                               curvepars=c(10,1000000000),
                               evopars=c(0,psi_index,0,1),
                               covars = NULL, #when covars is not specified, identity matrix is used, which implies 1 for product of mutation rate and inverse of var in # of offspring.
                              # covars=mutrate,  
                               parameters = pars.spec,
                               options=NULL,
                               clean=TRUE)
colnames(canonicalpsi_CA$curvepoints)[2:4]<-Env
colnames(canonicalpsi_CA$curvepoints)[6:27]<-output 
canonicalpsi = as.data.frame(canonicalpsi_CA$curvepoints)

canonicalpsi$fraction <-(canonicalpsi$In2_L2+canonicalpsi$In2_J+canonicalpsi$In2_A)/(canonicalpsi$In2_L2+canonicalpsi$In2_J+canonicalpsi$In2_A+canonicalpsi$In1_L2+canonicalpsi$In1_J+canonicalpsi$In1_A) 

canonicalpsi$Time = canonicalpsi$Evol.time/mutrate 

ggplot(data=canonicalpsi,aes(x=Time,y=PSI))+geom_path()+ylim(c(0,1))
ggplot(data=canonicalpsi,aes(x=Time,y=fraction))+geom_path()+ylim(c(0,1))


#
################
####Code for Figure 3: Evolution of metamorphosis#####
##Attack rates full meta
attackrates(Xj=1.742,meta=1,psi=0,sizedata=sizedata,dataframe='attackmeta')
ggplot(data=attackmeta,aes(x=size,y=a1pre))+geom_line((aes(colour='a1')),size=1)+
  geom_line(aes(y=a1post,colour='a1'),size=1)+
  geom_line(aes(y=a2pre,colour='a2'),size=1)+geom_line(aes(y=a2post,colour='a2'),size=1)
##Canonical equation
mutrate = 1##=mutrate*inverse of offspring variance *var in mut
var = 1

#Initial no specialization
Xmax2 = 10
psi = 0.0847351

#Initial specialization is
Xmax2 = 2
psi = 0.889465

meta_init=0
Xj_init=1.5 #In model specification, irreversible mass at meta = xmin+xj. 
pars.meta1=c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
             0.742,1,qs,0.01,10,psi,meta_init,rho)
#Step 1: Starting from the trivial equi, find the bp from where to continue the non-trivial equilibrium 
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars.meta1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:4)]
#Step 2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.meta1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.meta1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step 3: Canonical equation of adaptive dynamics
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
canonical_all_CA <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta_init,Xj_init), 
                              curvepars=c(100,10000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = NULL, #when covars is not specified, identity matrix is used, which implies 1 for product of mutation rate and inverse of var in # of offspring. 
                              # covars=mutrate, 
                              parameters = pars.meta1,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_all_CA$curvepoints)[2:4]<-Env
colnames(canonical_all_CA$curvepoints)[8:29]<-output
canonicalmeta<-as.data.frame(canonical_all_CA$curvepoints)
canonicalmeta$Time= canonicalmeta$Evol.time/mutrate

ggplot(data=canonicalmeta,aes(x=Time,y=Meta))+geom_path(colour="red")+
  geom_path(aes(y=PSI),colour="blue")+geom_path(aes(y=Meta+PSI),colour="green")


################
####Code for Figure 4 and 5 and histeresis########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (B) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
Xj=0.001
meta=0
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(Xmax2,startpnt),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20), 
                    parameters=pars_p1,
                    options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                    clean=TRUE)
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
canonical_fig4 <- PSPMevodyn(modelname="Metamorphosis_model", 
                               startpoint=c(startpoint, psi,meta,Xj), 
                               curvepars=c(1,10000),
                               evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                               covars = NULL,
                               parameters = pars_p2,
                               options=NULL,
                               clean=TRUE)
colnames(canonical_fig4$curvepoints)[2:4]<-Env
colnames(canonical_fig4$curvepoints)[8:29]<-output
canonicalmeta_fig4<-as.data.frame(canonical_fig4$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_fig4$PSI[nrow(canonicalmeta_fig4)])
meta=canonicalmeta_fig4$Meta[nrow(canonicalmeta_fig4)]
Xj = canonicalmeta_fig4$Xj[nrow(canonicalmeta_fig4)]
startpnt=as.numeric(canonicalmeta_fig4[nrow(canonicalmeta_fig4), c(2:4)])
canonical_fig4B <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpnt, psi,meta,Xj), 
                             curvepars=c(100,1000000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_fig4B$curvepoints)[2:4]<-Env
colnames(canonical_fig4B$curvepoints)[8:29]<-output
canonicalmeta_fig4B<-as.data.frame(canonical_fig4B$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_fig4B$PSI[nrow(canonicalmeta_fig4B)])
meta=canonicalmeta_fig4B$Meta[nrow(canonicalmeta_fig4B)]
Xj = canonicalmeta_fig4B$Xj[nrow(canonicalmeta_fig4B)]
startpnt=as.numeric(canonicalmeta_fig4B[nrow(canonicalmeta_fig4B), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='ESS',
                   startpoint=c(Xmax2,startpnt,meta,Xj ),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                   parameters=pars_p2,
                   options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                   clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$ESS = ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS2_data$ESS = ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?"))))
#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
MetaESS3_data$Meta=1
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
        geom_point(data=MetaESS2_data,aes(colour=ESS))+
        geom_point(data=MetaESS3_data,aes(colour=ESS))
#Use Canonical equation to find to which xj values it evolves in bistable area. 
#Even though selection gradient of psi is sometimes positive, it seems to be numerical error, it does not really evolve away from zero and stays very small
#Example
Xmax2=1.0079158
Meta=1
Psi=0
start=c(4.8309796, 0.9153785, 2.588088e-08, Psi, Meta, 1.7637535)
pars_test =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
Canonicaltest <- PSPMevodyn(modelname="Metamorphosis_model", 
                            startpoint=start, 
                            curvepars=c(0.1,1000000),
                            evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                            covars = NULL,
                            parameters = pars_test,
                            options=NULL,
                            clean=TRUE)
#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS3_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-10,]
meta=0.9
psi=0
Xj=pnt$Xj
Xmax2=pnt$R2max
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(1,1,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(meta,startpnt),
                   stepsize=-0.1, 
                   parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                   parameters=pars_inst,
                   options=c("popEVO", "0"),
                   clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
View(FindERP$curvepoints)
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpoint,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_inst2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
ggplot(data=as.data.frame(MetaESS_Inst$curvepoints),aes(x=R2max,y=Xj))+geom_path()+
  geom_path(data=as.data.frame(MetaESS_Inst2$curvepoints))
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure B1####
meta=0
Xj=0.001
psi=0
Xmax1=5
Xmax2=0.1
xmin=1
rho=0.5
qs=0.2
alpha=0.6
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Bif over Xmax2 to see where psi will evolve away from zero
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoPsi <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='EQ',
                  startpoint=c(Xmax2,startpnt),
                  stepsize=0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                  parameters=pars.spec,
                  options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                  clean=TRUE)
##Continue ESS from ERP point
ESSstart=as.data.frame(NoPsi$bifpoints)
Xmax2=NoPsi$bifpoints[1]
startpoint=as.numeric(c(ESSstart[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS<-PSPMequi(modelname="Metamorphosis_model",
                  biftype='ESS',
                  startpoint=c(Xmax2,startpoint,0 ),
                  stepsize=-0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, psi_index,0,1), 
                  parameters=pars.spec,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PSI_ESS_data=as.data.frame(PSI_ESS$curvepoints)
PSI_ESS_data$R0_xx_yy = PSI_ESS_data$`R0_xx[24]` - PSI_ESS_data$`R0_yy[24]`
PSI_ESS_data$ESS = ifelse(is.na(PSI_ESS_data$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))
#Numerical test that erp is css
####Code for Figure C1#####
#With use of canonical equation many values of psi and x2max tested
psi=0.5 #or different
Xmax2=5 #or different
Xj=0.001
meta=0
Xmax1=5
xmin=1
rho=0.5
qs=0.2
alpha=0.6
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Canonical equation
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
Canon<-PSPMevodyn(modelname="Metamorphosis_model", 
                  startpoint=c(startpnt, psi,0,Xj),
                  curvepars=c(1,1000000),
                  evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                  covars = NULL,
                  parameters = pars.spec,
                  options=NULL,
                  clean=TRUE)
####Code for Figure D1A####
meta=0
Xj=0.001
psi=0
Xmax1=2
Xmax2=0.1
xmin=1
rho=0.5
qs=0.2
alpha=0.6
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.01, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Bif over Xmax2 to see where psi will evolve away from zero
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoPsi <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='EQ',
                  startpoint=c(Xmax2,startpnt),
                  stepsize=0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30), 
                  parameters=pars.spec,
                  options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                  clean=TRUE)
##Continue ESS from ERP point
ESSstart=as.data.frame(NoPsi$bifpoints)[1,]
Xmax2=NoPsi$bifpoints[1,1]
startpoint=as.numeric(c(ESSstart[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS<-PSPMequi(modelname="Metamorphosis_model",
                  biftype='ESS',
                  startpoint=c(Xmax2,startpoint,0 ),
                  stepsize=-0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, psi_index,0,1), 
                  parameters=pars.spec,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PSI_ESS_data=as.data.frame(PSI_ESS$curvepoints)
PSI_ESS_data$R0_xx_yy = PSI_ESS_data$`R0_xx[24]` - PSI_ESS_data$`R0_yy[24]`
PSI_ESS_data$ESS = ifelse(is.na(PSI_ESS_data$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))
#Continue CSS
ESSstart2=as.data.frame(NoPsi$bifpoints)[2,]
Xmax2_2=NoPsi$bifpoints[2,1]
startpoint=as.numeric(c(ESSstart2[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2_2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS2<-PSPMequi(modelname="Metamorphosis_model",
                   biftype='ESS',
                   startpoint=c(Xmax2_2,startpoint,0 ),
                   stepsize=-0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30,0, psi_index,0,1), 
                   parameters=pars.spec,
                   options=c("popEVO", "0"),
                   clean=TRUE)
PSI_ESS_data2=as.data.frame(PSI_ESS2$curvepoints)
PSI_ESS_data2$R0_xx_yy = PSI_ESS_data2$`R0_xx[24]` - PSI_ESS_data2$`R0_yy[24]`
PSI_ESS_data2$ESS = ifelse(is.na(PSI_ESS_data2$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data2$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data2$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))+
  geom_point(data=PSI_ESS_data2,aes(colour=ESS))

####Code for Figure D1B####
meta=0
Xj=0.001
psi=0
Xmax1=10
Xmax2=0.1
xmin=1
rho=0.5
qs=0.2
alpha=0.6
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Bif over Xmax2 to see where psi will evolve away from zero
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoPsi <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='EQ',
                  startpoint=c(Xmax2,startpnt),
                  stepsize=0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30), 
                  parameters=pars.spec,
                  options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                  clean=TRUE)
##Continue ESS from ERP point
ESSstart=as.data.frame(NoPsi$bifpoints)[1,]
Xmax2=NoPsi$bifpoints[1,1]
startpoint=as.numeric(c(ESSstart[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS<-PSPMequi(modelname="Metamorphosis_model",
                  biftype='ESS',
                  startpoint=c(Xmax2,startpoint,0 ),
                  stepsize=-0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, psi_index,0,1), 
                  parameters=pars.spec,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PSI_ESS_data=as.data.frame(PSI_ESS$curvepoints)
PSI_ESS_data$R0_xx_yy = PSI_ESS_data$`R0_xx[24]` - PSI_ESS_data$`R0_yy[24]`
PSI_ESS_data$ESS = ifelse(is.na(PSI_ESS_data$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))


####Code for Figure D1C########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=2 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (B) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.01, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,10000000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,10000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = NULL,
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))
#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Xj))+geom_path()
#Use Canonical equation to find to which xj values it evolves in bistable area. 
#Even though selection gradient of psi is sometimes positive, it seems to be numerical error, it does not really evolve away from zero
#Example
pnt = MetaESS3_data[nrow(MetaESS3_data)-5,]
Xmax2=pnt$R2max
Meta=1
Xj=pnt$Xj
Psi=0
start=as.numeric(c(pnt[2:4],Psi, Meta,Xj))
pars_test =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
Canonicaltest <- PSPMevodyn(modelname="Metamorphosis_model", 
                            startpoint=start, 
                            curvepars=c(1,1000000),
                            evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                            covars = NULL,
                            parameters = pars_test,
                            options=NULL,
                            clean=TRUE)
#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS3_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-25,]
Xmax2=pnt$R2max
meta=0.6
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.01, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
ggplot(data=as.data.frame(MetaESS_Inst$curvepoints),aes(x=R2max,y=Meta))+geom_path()+
  geom_path(data=as.data.frame(MetaESS_Inst2$curvepoints))
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure D1D########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=10 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (B) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(1,1,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
psi=0
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(1,10000000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Bistability, Find other equilibrium
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj , 5,
           0.742,1,qs,0.01,5,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(1,1,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmax1), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
OtherEqui=as.data.frame(Nontriv$curvepoints)
#Continue in other equi
startpnt=as.numeric(OtherEqui[nrow(OtherEqui), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(10,1000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = NULL,
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)
#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))
#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3<- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.01, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 2.68842647E+00, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Xj))+geom_path()
#Use Canonical to find ESS point after lp
#Start left of lp
index=which.min(MetaESS3_data$R2max)
pnt = MetaESS3_data[index,]
Xmax2=pnt$R2max-0.1
meta=1
Xj=pnt$Xj
Psi=0
start=as.numeric(c(pnt[2:4],Psi, Meta,Xj))
pars_test =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(1,1,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_test,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_test[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_test,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Canonical equation to find ess. Meta first decreases but then increases again. 
start=as.numeric(Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)])
Canonicaltest <- PSPMevodyn(modelname="Metamorphosis_model", 
                            startpoint=c(start, Psi,meta,Xj),
                            curvepars=c(10,100000000),
                            evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                            covars = NULL,
                            parameters = pars_test,
                            options=NULL,
                            clean=TRUE)
pnt=as.numeric(Canonicaltest$curvepoints[nrow(Canonicaltest$curvepoints),c(2:4)])
meta=min(1,CanData$Meta[nrow(CanData)])
Xj=CanData$Xj[nrow(CanData)]
psi=0

pars_p4 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,0.5)
MetaESS4<- PSPMequi(modelname="Metamorphosis_model",
                    biftype='ESS',
                    startpoint=c(Xmax2,pnt,Xj ),
                    stepsize=-0.01, 
                    parbnds=c(which(pars.names == 'Xmax2') - 1, 0, 20, 0, xj_index,0.00001,20), 
                    parameters=pars_p4,
                    options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                    clean=TRUE)
colnames(MetaESS4$curvepoints)[2:4]<-Env
colnames(MetaESS4$curvepoints)[6:27]<-output
MetaESS4_data=as.data.frame(MetaESS4$curvepoints)
MetaESS4_data$R0_xx_yy = MetaESS4_data$`R0_xx[17]` - MetaESS4_data$`R0_yy[17]`
MetaESS4_data$ESS = ifelse(is.na(MetaESS4_data$R0_xx_yy), 'ESS', ifelse(MetaESS4_data$R0_xx_yy<0, "ERP", ifelse(MetaESS4_data$`R0_yy[17]`<0, "CSS", "EBP")))
#Selection gradient of meta becomes negative at low r2max values
#Use canonical equation to determine evolution after this point -> meta to zero
meta_away=subset(MetaESS4_data, `R0_x[25]`<0)
meta=1
Xj=meta_away$Xj[1]
Psi=0
Xmax2=meta_away$R2max[1]
start=as.numeric(meta_away[1,c(2:4)])
pars_p5 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,0.5)
Canonicalrun <- PSPMevodyn(modelname="Metamorphosis_model", 
                            startpoint=pnt,#c(start, Psi,meta,Xj),
                            curvepars=c(100,100000000),
                            evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                            covars = NULL,
                            parameters = pars_p5,
                            options=NULL,
                            clean=TRUE)

#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS4_data, Birthrate>0&`R0_x[25]`>=0)
pnt=PosBirth[nrow(PosBirth)-10,]
Xmax2=pnt$R2max
meta=0.9
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)


####Code for Figure D2A####
meta=0
Xj=0.001
psi=0
Xmax1=5
Xmax2=0.1
xmin=0.5
rho=0.5
qs=0.2
alpha=0.6
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Bif over Xmax2 to see where psi will evolve away from zero
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoPsi <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='EQ',
                  startpoint=c(Xmax2,startpnt),
                  stepsize=0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30), 
                  parameters=pars.spec,
                  options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                  clean=TRUE)
##Continue ESS from ERP point
ESSstart=as.data.frame(NoPsi$bifpoints)[1,]
Xmax2=NoPsi$bifpoints[1,1]
startpoint=as.numeric(c(ESSstart[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS<-PSPMequi(modelname="Metamorphosis_model",
                  biftype='ESS',
                  startpoint=c(Xmax2,startpoint,0 ),
                  stepsize=-0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, psi_index,0,1), 
                  parameters=pars.spec,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PSI_ESS_data=as.data.frame(PSI_ESS$curvepoints)
PSI_ESS_data$R0_xx_yy = PSI_ESS_data$`R0_xx[24]` - PSI_ESS_data$`R0_yy[24]`
PSI_ESS_data$ESS = ifelse(is.na(PSI_ESS_data$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))
#Continue CSS
ESSstart2=as.data.frame(NoPsi$bifpoints)[4,]
Xmax2_2=NoPsi$bifpoints[4,1]
startpoint=as.numeric(c(ESSstart2[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2_2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS2<-PSPMequi(modelname="Metamorphosis_model",
                   biftype='ESS',
                   startpoint=c(Xmax2_2,startpoint,0 ),
                   stepsize=-0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30,0, psi_index,0,1), 
                   parameters=pars.spec,
                   options=c("popEVO", "0"),
                   clean=TRUE)
PSI_ESS_data2=as.data.frame(PSI_ESS2$curvepoints)
PSI_ESS_data2$R0_xx_yy = PSI_ESS_data2$`R0_xx[24]` - PSI_ESS_data2$`R0_yy[24]`
PSI_ESS_data2$ESS = ifelse(is.na(PSI_ESS_data2$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data2$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data2$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))+
  geom_point(data=PSI_ESS_data2,aes(colour=ESS))

####Code for Figure D2B####
meta=0
Xj=0.001
psi=0
Xmax1=5
Xmax2=0.1
xmin=3.44
rho=0.5
qs=0.2
alpha=0.6
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Bif over Xmax2 to see where psi will evolve away from zero
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoPsi <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='EQ',
                  startpoint=c(Xmax2,startpnt),
                  stepsize=.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 12), 
                  parameters=pars.spec,
                  options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                  clean=TRUE)
NoPsi$biftypes
##Continue ESS from ERP point
ESSstart=as.data.frame(NoPsi$bifpoints)[1,]
Xmax2=NoPsi$bifpoints[1,1]
startpoint=as.numeric(c(ESSstart[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS<-PSPMequi(modelname="Metamorphosis_model",
                  biftype='ESS',
                  startpoint=c(Xmax2,startpoint,0 ),
                  stepsize=-0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, psi_index,0,1), 
                  parameters=pars.spec,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PSI_ESS_data=as.data.frame(PSI_ESS$curvepoints)
PSI_ESS_data$R0_xx_yy = PSI_ESS_data$`R0_xx[24]` - PSI_ESS_data$`R0_yy[24]`
PSI_ESS_data$ESS = ifelse(is.na(PSI_ESS_data$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))

####Code for Figure D2C########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=0.5 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (B) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.3,0.3,Xmax2), 
                 stepsize=0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.5
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.001
psi=0
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,10000000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = c(1,0,0,0,1,0,0,0,0.00001), #let xj evolve slow, otherwise numerical errors
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Bistability, Find other equilibrium
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj , 5,
           0.742,1,qs,0.01,5,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.35,0.35,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmax1), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)

OtherEqui=as.data.frame(Nontriv$curvepoints)
#Continue in other equi
startpnt=as.numeric(OtherEqui[nrow(OtherEqui), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,100000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = c(1,0,0,0,1,0,0,0,.01),
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)

colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$ESS = ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0, 'CSS', ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))
#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Xj))+geom_path()

#Use Canonical equation to find to which xj values it evolves in bistable area. 
#Even though selection gradient of psi is sometimes positive, it seems to be numerical error, it does not really evolve away from zero
#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS3_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-7,]
Xmax2=pnt$R2max
meta=0.7
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure D2D########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=3.44 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (B) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 = c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.6,0.6,Xmax2), 
                 stepsize=0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 15), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.5
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,10000000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,10000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = NULL,
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$ESS = ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0, 'CSS', ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))
#Continue backwards calculation with Meta = 1 
bla =readRDS("metaESS3_figD3D.RDS")
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Xj))+geom_path()
#Use Canonical equation to find to which xj values it evolves in bistable area. 
#Even though selection gradient of psi is sometimes positive, it seems to be numerical error, it does not really evolve away from zero

#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS3_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-5,]
Xmax2=pnt$R2max
meta=0.95
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=-0.01, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
ggplot(data=as.data.frame(MetaESS_Inst$curvepoints),aes(x=R2max,y=Meta))+geom_path()+
  geom_path(data=as.data.frame(MetaESS_Inst2$curvepoints))
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure D3A########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.9 #0 (B) or 0.9 (A) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.5
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,1000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,1000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = NULL,
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)
#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))
#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS2_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-5,]
Xmax2=pnt$R2max
meta=0.9
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(4,4,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=-0.01, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
ggplot(data=as.data.frame(MetaESS_Inst$curvepoints),aes(x=R2max,y=Meta))+geom_path()+
  geom_path(data=as.data.frame(MetaESS_Inst2$curvepoints))
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure D3B########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0 #0 (B) or 0.9 (A) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,10000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,100000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = c(1,0,0,0,1,0,0,0,0.0001),
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta=1 equilibrium
#Step C1: Equilibrium with Xj in ESS, Psi and meta not since it is at its minimum/maximum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=min(1,canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)])
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$R0_xx_yy = MetaESS1_data$`R0_xx[17]` - MetaESS1_data$`R0_yy[17]`
MetaESS1_data$ESS = ifelse(is.na(MetaESS1_data$R0_xx_yy), 'ESS', ifelse(MetaESS1_data$R0_xx_yy<0, "ERP", ifelse(MetaESS1_data$`R0_yy[17]`<0, "CSS", "EBP")))
MetaESS1_data=subset(MetaESS1_data,`R0_x[25]`>0)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS2_data$R0_xx_yy = MetaESS2_data$`R0_xx[17]` - MetaESS2_data$`R0_yy[17]`
MetaESS2_data$ESS = ifelse(is.na(MetaESS2_data$R0_xx_yy), 'ESS', ifelse(MetaESS2_data$R0_xx_yy<0, "ERP", ifelse(MetaESS2_data$`R0_yy[17]`<0, "CSS", "EBP")))

#Use Canonical equation to find to which xj values it evolves in bistable area. 
#Even though selection gradient of psi is sometimes positive, it seems to be numerical error, it does not really evolve away from zero

#Continue forwards calculation with Meta != 1 
endpoint=MetaESS1_data[nrow(MetaESS1_data),]
meta=min(1,endpoint$Meta)-0.0001
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$ESS=ifelse(MetaESS3_data$`eig J`>0, "ERP", ifelse(MetaESS3_data$`eig H`<0&MetaESS3_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS3_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS3_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 


#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS2_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-2,]
Xmax2=pnt$R2max
meta=0.85
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.01, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure D3C########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (A) for fig D3
qs = 0 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,10000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,1000000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = c(1,0,0,0,1,0,0,0,0.0001),#xj slower to avoid numerical issues
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))

#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))

#Use Canonical equation to find to which xj values it evolves in bistable area. 
#Even though selection gradient of psi is sometimes positive, it seems to be numerical error, it does not really evolve away from zero

#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS3_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-3,]
Xmax2=pnt$R2max
meta=0.9
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(1,1,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=0.001, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
#Use canonical equation to determine erp for low values of r2max. 

####Code for Figure D3D######## 
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (A) for fig D3
qs = 0.742 #0 (C) or 0.742(D) for fig D3
alpha = 0.6 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Xj_init=0.0001
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj_init, 5,
           0.742,1,qs,0.01,10,psi,meta_init,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta_init,Xj_init), 
                             curvepars=c(10,10000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
##Increase stepsize##
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]

startpnt <- canonicalmeta_figD[nrow(canonicalmeta_figD),c(2:4)]
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,100000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = c(1,0,0,0,1,0,0,0,0.1),#xj slower to avoid numerical issues, ess does not change when this is back to one (and timestep very small) 
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))

#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Xj))+geom_point(aes(colour=ESS))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Birthrate))+geom_point(aes(colour=ESS))
###IN ERP cycling around xj, meta & psi sometimes smaller/larger than 1/0, but very small change. Not relevant
#What happens after xj = xf?
endpoint=MetaESS3_data[nrow(MetaESS3_data),]
Xmax2=as.numeric(endpoint[1])-0.1
pnt=as.numeric(endpoint[c(2:4)])
Xj=4.005
meta=1
psi=0
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)

BPfind<-PSPMequi(modelname="Metamorphosis_model2", 
                 biftype="EQ", 
                 startpoint=c(0.7,0.7,Xmax2), 
                 stepsize=0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p3,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model2",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p3,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
pnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
Canonical_test <- PSPMevodyn(modelname="Metamorphosis_model2", 
                             startpoint=c(pnt, psi,meta,Xj), 
                             curvepars=c(100,100000000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = c(1,0,0,0,1,0,0,0,1),
                             parameters = pars_p3,
                             options=NULL,
                             clean=TRUE)


env = c(8.40979495E-01, 9.57567513E-01, 5.09890926E-07 )
Xj=4.58461855E+00
MetaESS4 <- PSPMequi(modelname="Metamorphosis_model2",
                     biftype='ESS',
                     startpoint=c(Xmax2,env,Xj ),
                     stepsize=-0.01, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS4$curvepoints)[2:4]<-Env
colnames(MetaESS4$curvepoints)[6:27]<-output
MetaESS4_data=as.data.frame(MetaESS4$curvepoints)
MetaESS4_data$R0_xx_yy = MetaESS4_data$`R0_xx[17]` - MetaESS4_data$`R0_yy[17]`
MetaESS4_data$ESS = ifelse(is.na(MetaESS4_data$R0_xx_yy), 'ESS', ifelse(MetaESS4_data$R0_xx_yy<0, "ERP", ifelse(MetaESS4_data$`R0_yy[17]`<0, "CSS", "EBP")))

MetaESS5 <- PSPMequi(modelname="Metamorphosis_model2",
                     biftype='ESS',
                     startpoint=c(Xmax2,env,Xj ),
                     stepsize=0.01, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS5$curvepoints)[2:4]<-Env
colnames(MetaESS5$curvepoints)[6:27]<-output
MetaESS5_data=as.data.frame(MetaESS5$curvepoints)
MetaESS5_data$R0_xx_yy = MetaESS5_data$`R0_xx[17]` - MetaESS5_data$`R0_yy[17]`
MetaESS5_data$ESS = ifelse(is.na(MetaESS5_data$R0_xx_yy), 'ESS', ifelse(MetaESS5_data$R0_xx_yy<0, "ERP", ifelse(MetaESS5_data$`R0_yy[17]`<0, "CSS", "EBP")))


###Little bit of bistability in XJ
ggplot(data=MetaESS1_data,aes(x=R2max,y=(Xj+1)*1.742))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))+
  geom_point(data=MetaESS3_data,aes(colour=ESS))+
  geom_point(data=MetaESS4_data,aes(colour=ESS))+
  geom_point(data=MetaESS5_data,aes(colour=ESS))+
  geom_hline(aes(yintercept=5*1.742))+
  coord_cartesian(xlim=c(0,1.2))

###What is the maximum size individuals reach?#
#Simulate individual life history to figure out#
meta=1
psi=0
nr=37
Xmax2=MetaESS4_data[nr,1]
pnt=as.numeric(MetaESS4_data[nr,c(2:4)])
Xj=MetaESS4_data[nr,5]

pars_indiv =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)

indiv_calc<-PSPMind(modelname="Metamorphosis_model2",
        environment=pnt,
        parameters=pars_indiv)
#View(indiv_calc$Pop00)
maxsize=as.numeric(indiv_calc$Pop00[nrow(indiv_calc$Pop00),3]+indiv_calc$Pop00[nrow(indiv_calc$Pop00),4])
Xmax2
maxsize

Sizedata=data.frame(maxsize=c(12.24624,11.59812,11.85842, 12.10544,10.53207,10.81976,
                              10.32999, 11.35452,11.19335, 11.19463, 11.27468,11.37329),
                    X2max=c( 1.0618,0.9880331,1.015204, 1.043898,0.8996039,  0.9206282,
                             0.8852939,0.9649421,1.064942,1.065891,1.134456, 1.243223))

##Xj goes to size that no-one reaches


#Step D: Find unstable equilibrium
Xmax2=NoMetaData[nrow(NoMetaData),1]-0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)])
meta=0
Xj=0.0001
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpoint),
                    stepsize=0.1,
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1),
                    parameters=pars_inst,
                    options=c("popEVO", "0", 'parEVO',as.character(xj_index),
                              "popEVO", "0", 'parEVO',as.character(meta_index)),
                    clean=TRUE)
FindERP$biftypes
View(FindERP$curvepoints)

meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[1,2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index, 0, 1), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                                   "popEVO", "0", 'parEVO',as.character(xj_index)),
                         clean=TRUE)
View(MetaESS_Inst$curvepoints)
inst=as.data.frame((MetaESS_Inst$curvepoints))


##Instable equi is for xj=0.0001, check with canonical if this is correct
Meta_pnt=0.32
Xmax2_pnt=2.5
Xj=3
ggplot(data=inst,aes(x=R2max,y=Meta))+geom_path()+geom_point(aes(x=Xmax2_pnt,y=Meta_pnt),colour='red')

pars_p1 =c(0.1, Xmax1, Xmax2_pnt, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,Meta_pnt,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
# Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Canonical
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
test<- PSPMevodyn(modelname="Metamorphosis_model", 
              startpoint=c(startpnt, psi,Meta_pnt,Xj), 
              curvepars=c(100,100000000),
              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
              covars = c(1,0,0,0,1,0,0,0,1),
              parameters = pars_p1,
              options=NULL,
              clean=TRUE)


####Code for Figure D4A####
meta=0
Xj=0.001
psi=0
Xmax1=5
Xmax2=0.1
xmin=1
rho=0.5
qs=0.2
alpha=0.93
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.spec,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.spec,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Bif over Xmax2 to see where psi will evolve away from zero
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoPsi <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='EQ',
                  startpoint=c(Xmax2,startpnt),
                  stepsize=0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30), 
                  parameters=pars.spec,
                  options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                  clean=TRUE)
##Continue ESS from ERP point
ESSstart=as.data.frame(NoPsi$bifpoints)[1,]
Xmax2=NoPsi$bifpoints[1,1]
startpoint=as.numeric(c(ESSstart[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS<-PSPMequi(modelname="Metamorphosis_model",
                  biftype='ESS',
                  startpoint=c(Xmax2,startpoint,0 ),
                  stepsize=-0.1, 
                  parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, psi_index,0,1), 
                  parameters=pars.spec,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PSI_ESS_data=as.data.frame(PSI_ESS$curvepoints)
PSI_ESS_data$R0_xx_yy = PSI_ESS_data$`R0_xx[24]` - PSI_ESS_data$`R0_yy[24]`
PSI_ESS_data$ESS = ifelse(is.na(PSI_ESS_data$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))
#Continue CSS
ESSstart2=as.data.frame(NoPsi$bifpoints)[2,]
Xmax2_2=NoPsi$bifpoints[2,1]
startpoint=as.numeric(c(ESSstart2[2:4]))
pars.spec=c(0.1, Xmax1, Xmax2_2, 10000,100000,
            alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
            0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
            0.742,1,qs,0.01,10,psi,meta,rho)
PSI_ESS2<-PSPMequi(modelname="Metamorphosis_model",
                   biftype='ESS',
                   startpoint=c(Xmax2_2,startpoint,0 ),
                   stepsize=-0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30,0, psi_index,0,1), 
                   parameters=pars.spec,
                   options=c("popEVO", "0"),
                   clean=TRUE)
PSI_ESS_data2=as.data.frame(PSI_ESS2$curvepoints)
PSI_ESS_data2$R0_xx_yy = PSI_ESS_data2$`R0_xx[24]` - PSI_ESS_data2$`R0_yy[24]`
PSI_ESS_data2$ESS = ifelse(is.na(PSI_ESS_data2$R0_xx_yy), 'ESS', ifelse(PSI_ESS_data2$R0_xx_yy<0, "ERP", ifelse(PSI_ESS_data2$`R0_yy[24]`<0, "CSS", "EBP")))
ggplot(data=PSI_ESS_data,aes(x=R2max,y=PSI))+geom_point(aes(colour=ESS))+
  geom_point(data=PSI_ESS_data2,aes(colour=ESS))

####Code for Figure D4B########
###Step A: Non-Meta equilibrium: Find point where metamorphosis can evolve
#Step A1: Find non-triv equi for low value of Xmax2
Xmax2 = 0.1 
psi = 0
Xmax1=5 #2 (C) or 10 (D) for fig D1
xmin=1 # 0.5 (C) or 3.44 (D) for fig D2
rho=0.5 #0 (B) or 0.9 (A) for fig D3
qs = 0.2 #0 (C) or 0.742(D) for fig D3
alpha = 0.93 #0.93 for fig D4
meta = 0
psi = 0
Xj = 0.001
pars_p1 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.01,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_p1,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step A2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_p1[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_p1,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step A3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20), 
                   parameters=pars_p1,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[25]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.5
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 

pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
canonical_figD <- PSPMevodyn(modelname="Metamorphosis_model", 
                             startpoint=c(startpoint, psi,meta,Xj), 
                             curvepars=c(10,10000),
                             evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                             covars = NULL,
                             parameters = pars_p2,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:29]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
psi = max(0,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
meta=canonicalmeta_figD$Meta[nrow(canonicalmeta_figD)]
Xj = canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)]
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Metamorphosis_model", 
                              startpoint=c(startpnt, psi,meta,Xj), 
                              curvepars=c(100,10000000),
                              evopars=c(0,psi_index,0,1, 0, meta_index,0,1,0,xj_index,0,20),
                              covars = c(1,0,0,0,1,0,0,0,0.0001),#xj slower to avoid numerical issues
                              parameters = pars_p2,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:29]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
psi = max(0,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
meta=canonicalmeta_figDB$Meta[nrow(canonicalmeta_figDB)]
Xj = canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)]
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
pars_p2 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,0.2,0.01,10,psi,meta,rho)
MetaESS1 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,meta,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                     parameters=pars_p2,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:28]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:28]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
ggplot(data=MetaESS1_data,aes(x=R2max,y=Meta))+geom_point(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))
#Continue backwards calculation with Meta = 1 
endpoint=MetaESS2_data[nrow(MetaESS2_data),]
meta=min(1,endpoint$Meta)
Xj=as.numeric(endpoint$Xj)
startpnt=as.numeric(endpoint[2:4])
Xmax2=as.numeric(endpoint[1])
pars_p3 =c(0.1, Xmax1, Xmax2, 10000,100000,
           alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
           0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
           0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS3 <- PSPMequi(modelname="Metamorphosis_model",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Xj ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20, 0, xj_index,0.00001,20), 
                     parameters=pars_p3,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),"popEVO", "0", 'parEVO',as.character(meta_index)),
                     clean=TRUE)
colnames(MetaESS3$curvepoints)[2:4]<-Env
colnames(MetaESS3$curvepoints)[6:27]<-output
MetaESS3_data=as.data.frame(MetaESS3$curvepoints)
MetaESS3_data$R0_xx_yy = MetaESS3_data$`R0_xx[17]` - MetaESS3_data$`R0_yy[17]`
MetaESS3_data$ESS = ifelse(is.na(MetaESS3_data$R0_xx_yy), 'ESS', ifelse(MetaESS3_data$R0_xx_yy<0, "ERP", ifelse(MetaESS3_data$`R0_yy[17]`<0, "CSS", "EBP")))
ggplot(data=MetaESS3_data,aes(x=R2max,y=Xj))+geom_path()

#Step D: Find unstable equilibrium
#Step D1: Find an equilibrium close to the bp, where meta =! 1 Use point closer/further away from bp if it doesnt work
PosBirth=subset(MetaESS3_data,Birthrate>0)
pnt=PosBirth[nrow(PosBirth)-3,]
Xmax2=pnt$R2max
meta=0.9
psi=0
Xj=pnt$Xj
pars_inst =c(0.1, Xmax1, Xmax2, 10000,100000,
             alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
             0.742,1,qs,0.01,10,psi,meta,rho)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=pars_inst,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars_inst[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.01, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step D2: Continue over meta to find ERP
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
FindERP <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(meta,startpnt),
                    stepsize=-0.01, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                    parameters=pars_inst,
                    options=c("popEVO", "0"),
                    clean=TRUE)
colnames(FindERP$curvepoints)[2:4]<-Env
colnames(FindERP$curvepoints)[5:(ncol(FindERP$curvepoints)-3)]<-output
colnames(FindERP$bifpoints)[2:4]<-Env
colnames(FindERP$bifpoints)[5:(ncol(FindERP$bifpoints)-3)]<-output
FindErpData=as.data.frame(FindERP$curvepoints)
#Step D3:USE ERP OR EBP (This is not an eBP because thtat depends on xj as well!) as startpoint
meta=FindERP$bifpoints[1]
startpoint=as.numeric(c(FindERP$bifpoints[2:4]))
pars_inst2 =c(0.1, Xmax1, Xmax2, 10000,100000,
              alpha, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
              0.033, 0.77,6.71E-06,0.5, 0.000804,xmin, Xj, 5,
              0.742,1,qs,0.01,10,psi,meta,rho)
MetaESS_Inst <- PSPMequi(modelname="Metamorphosis_model",
                         biftype='ESS',
                         startpoint=c(Xmax2,startpoint,meta,Xj ),
                         stepsize=-0.1, 
                         parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                         parameters=pars_inst2,
                         options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                         clean=TRUE)
MetaESS_Inst2 <- PSPMequi(modelname="Metamorphosis_model",
                          biftype='ESS',
                          startpoint=c(Xmax2,startpoint,meta,Xj ),
                          stepsize=0.1, 
                          parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 20,0, meta_index,0,1, 0, xj_index,0.00001,20), 
                          parameters=pars_inst2,
                          options=c("popEVO", "0", 'parEVO',as.character(psi_index)),
                          clean=TRUE)
#Use canonical equation to determine erp for low values of r2max. 





####Code for Figure F1####
xyline = data.frame(Meta=c(0,1),Meta_mut=c(0,1))
####PIP Low (Xmax2 = 1)###### 
Xmax1=5
Xmax2 =1
Xj = 3.073516   #CSS value
psi = 0  #CSS value
meta = 0.8
pars.PIP = c(0.1, Xmax1, Xmax2, 10000,100000,
             0.6, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,1, Xj, 5,
             0.742,1,0.2,0.01,10,psi,meta,0.5)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.PIP,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.PIP,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step3: Find ESS of Meta
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
ESSfind<- PSPMequi(modelname="Metamorphosis_model",
            biftype='EQ',
            startpoint=c(meta,startpnt),
            stepsize=0.1, 
            parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
            parameters=pars.PIP,
            options=c("popEVO", "0"),
            clean=TRUE)

meta_erp = ESSfind$bifpoints[1,1]
environ=as.numeric(ESSfind$bifpoints[1,c(2:4)])
PIP <- PSPMequi(modelname="Metamorphosis_model",
                biftype='PIP',
                startpoint=c(meta_erp,environ,meta_erp),
                stepsize=0.01, 
                parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                parameters=pars.PIP,
                options=c("popEVO", "0"),
                clean=TRUE)
PIP1=as.data.frame(PIP$curvepoints)
colnames(PIP1) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')
PIP_B <- PSPMequi(modelname="Metamorphosis_model",
                biftype='PIP',
                startpoint=c(meta_erp,environ,meta_erp),
                stepsize=-0.01, 
                parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                parameters=pars.PIP,
                options=c("popEVO", "0"),
                clean=TRUE)
PIP2=as.data.frame(PIP_B$curvepoints)
colnames(PIP2) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')

ggplot(data=PIP1,aes(x=Meta,y=Meta_mut))+geom_path()+geom_line(data=xyline)+
  geom_path(data=PIP2)
####PIP intermediate (Xmax2 = 2.8)###### 
Xmax1=5
Xmax2 =2.825692
Xj = 0.02074192   #CSS value
psi = 0  #CSS value
meta = 0.5
pars.PIP = c(0.1, Xmax1, Xmax2, 10000,100000,
             0.6, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,1, Xj, 5,
             0.742,1,0.2,0.01,10,psi,meta,0.5)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.PIP,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.PIP,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step3: Find ESS of Meta
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
ESSfind<- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(meta,startpnt),
                   stepsize=-0.01, 
                   parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                   parameters=pars.PIP,
                   options=c("popEVO", "0"),
                   clean=TRUE)

meta_erp = ESSfind$bifpoints[1,1]
environ=as.numeric(ESSfind$bifpoints[1,c(2:4)])
PIP <- PSPMequi(modelname="Metamorphosis_model",
                biftype='PIP',
                startpoint=c(meta_erp,environ,meta_erp),
                stepsize=0.01, 
                parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                parameters=pars.PIP,
                options=c("popEVO", "0"),
                clean=TRUE)
PIP1=as.data.frame(PIP$curvepoints)
colnames(PIP1) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')
PIP_B <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='PIP',
                  startpoint=c(meta_erp,environ,meta_erp),
                  stepsize=-0.01, 
                  parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                  parameters=pars.PIP,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PIP2=as.data.frame(PIP_B$curvepoints)
colnames(PIP2) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')

ggplot(data=PIP1,aes(x=Meta,y=Meta_mut))+geom_path()+geom_line(data=xyline)+
  geom_path(data=PIP2)
####PIP high (Xmax2 = 4###### 
Xmax1=5
Xmax2 =4
Xj = 0.01025069   #CSS value
psi = 0  #CSS value
meta = 1
pars.PIP = c(0.1, Xmax1, Xmax2, 10000,100000,
             0.6, 17.42, 4E-06 ,8.19E-05,0.68, 0.00115,
             0.033, 0.77,6.71E-06,0.5, 0.000804,1, Xj, 5,
             0.742,1,0.2,0.01,10,psi,meta,0.5)
BPfind<-PSPMequi(modelname="Metamorphosis_model", 
                 biftype="EQ", 
                 startpoint=c(5,5,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,25),
                 parameters=pars.PIP,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-pars.spec[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Metamorphosis_model",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=pars.PIP,
                    options=c("popEVO", "0"),
                    clean=TRUE)
#Step3: Find ESS of Meta
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
ESSfind<- PSPMequi(modelname="Metamorphosis_model",
                   biftype='EQ',
                   startpoint=c(meta,startpnt),
                   stepsize=-0.01, 
                   parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1), 
                   parameters=pars.PIP,
                   options=c("popEVO", "0"),
                   clean=TRUE)

meta_erp = ESSfind$bifpoints[2,1]
meta_css = ESSfind$bifpoints[1,1]
environ=as.numeric(ESSfind$bifpoints[1,c(2:4)])
environ2=as.numeric(ESSfind$bifpoints[2,c(2:4)])

PIP <- PSPMequi(modelname="Metamorphosis_model",
                biftype='PIP',
                startpoint=c(meta_erp,environ,meta_erp),
                stepsize=0.01, 
                parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                parameters=pars.PIP,
                options=c("popEVO", "0"),
                clean=TRUE)
PIP1=as.data.frame(PIP$curvepoints)
colnames(PIP1) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')
PIP_B <- PSPMequi(modelname="Metamorphosis_model",
                  biftype='PIP',
                  startpoint=c(meta_erp,environ,meta_erp),
                  stepsize=-0.01, 
                  parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                  parameters=pars.PIP,
                  options=c("popEVO", "0"),
                  clean=TRUE)
PIP2=as.data.frame(PIP_B$curvepoints)
colnames(PIP2) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')
PIP3 <- PSPMequi(modelname="Metamorphosis_model",
                biftype='PIP',
                startpoint=c(meta_css,environ2,meta_css),
                stepsize=0.01, 
                parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                parameters=pars.PIP,
                options=c("popEVO", "0"),
                clean=TRUE)
PIP3=as.data.frame(PIP3$curvepoints)
colnames(PIP3) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')
PIP4 <- PSPMequi(modelname="Metamorphosis_model",
                 biftype='PIP',
                 startpoint=c(meta_css,environ2,meta_css),
                 stepsize=-0.01, 
                 parbnds=c(which(pars.names == 'Meta') - 1, 0.0, 1, meta_index,0,1), 
                 parameters=pars.PIP,
                 options=c("popEVO", "0"),
                 clean=TRUE)
PIP4=as.data.frame(PIP4$curvepoints)
colnames(PIP4) <- c('Meta', "X1", 'X2', 'Birthrate', 'Meta_mut')


ggplot(data=PIP1,aes(x=Meta,y=Meta_mut))+geom_path()+geom_line(data=xyline)+
  geom_path(data=PIP2)+geom_path(data=PIP3)+geom_path(data=PIP4)


